@@ -1,7 +1,7 @@
-use std::mem::size_of;
+use std::mem::{size_of, size_of_val};
 use nix;
 use nix::sys::ioctl::ioctl_num_type;
-use libc::{c_int, c_uint, uint8_t, uint16_t, clockid_t, ioctl};
+use libc::{c_char, c_int, c_uint, uint8_t, uint16_t, int32_t, uint32_t, clockid_t, ioctl};
 
 pub use libc::{
     timeval,
@@ -150,6 +150,20 @@ impl ff_effect_union {
     }
 }
 
+#[repr(C)]
+#[derive(Copy, Clone)]
+pub struct repeat_settings {
+    pub delay: c_uint,
+    pub period: c_uint,
+}
+
+#[repr(C)]
+#[derive(Copy, Clone)]
+pub struct input_mt_request_layout<T: ?Sized = [int32_t]> {
+    pub code: uint32_t,
+    pub values: T,
+}
+
 ioctl! {
     /// get driver version
     read ev_get_version with b'E', 0x01; c_int
@@ -162,12 +176,12 @@ ioctl! {
 
 ioctl! {
     /// get repeat settings
-    read ev_get_rep with b'E', 0x03; [c_uint; 2]
+    read ev_get_rep with b'E', 0x03; repeat_settings
 }
 
 ioctl! {
     /// set repeat settings
-    write_ptr ev_set_rep with b'E', 0x03; [c_uint; 2]
+    write_ptr ev_set_rep with b'E', 0x03; repeat_settings
 }
 
 ioctl! {
@@ -192,17 +206,17 @@ ioctl! {
 
 ioctl! {
     /// get device name
-    read_buf ev_get_name with b'E', 0x06; uint8_t
+    read_buf ev_get_name with b'E', 0x06; c_char
 }
 
 ioctl! {
     /// get physical location
-    read_buf ev_get_phys with b'E', 0x07; uint8_t
+    read_buf ev_get_phys with b'E', 0x07; c_char
 }
 
 ioctl! {
     /// get unique identifier
-    read_buf ev_get_uniq with b'E', 0x08; uint8_t
+    read_buf ev_get_uniq with b'E', 0x08; c_char
 }
 
 ioctl! {
@@ -210,9 +224,10 @@ ioctl! {
     read_buf ev_get_prop with b'E', 0x09; uint8_t
 }
 
-ioctl! {
-    /// get MT slot values
-    read_buf ev_get_mtslots with b'E', 0x0a; uint8_t
+/// get MT slot values
+pub unsafe fn ev_get_mtslots(fd: c_int, buf: *mut input_mt_request_layout) -> nix::Result<i32> {
+    // for some reason this isn't _IORW?
+    convert_ioctl_res!(ioctl(fd, ior!(b'E', 0x0a, size_of_val(&*buf)) as ioctl_num_type, buf))
 }
 
 ioctl! {
